%
% function dname = hippomat2grid(varargin)
%
%   This code can be called from hippomat_gui2 or independently.
%
%   dname = output string containing name of subdirectory where results are
%
%   dt is hard-coded in hippomat_gui2
%   All other input variables are defined in the gui hippomat_gui2
%
%   To run without the gui: nargin = 11
%   varargin{1}: dt = time step (ms)
%   varargin{2}: Ne_M = number of exc. cells in the MEC
%   varargin{3}; Ne_C3 = number of exc. cells in the CA3
%   varargin{4}; Ni_C3 = number of inh. cells in the CA3
%   varargin{5}; SESSIONS = number of sessions to be run
%   varargin{6}: LAPS = number of laps per session
%   varargin{7}: pC3toC3 = plasticity rule for CA3 E to CA3 E weights
%       0: no plasticity
%       1: STDP from Song, Miller, and Abbott (2000)
%       2: STDP from Froemke and Dan (2002)
%   varargin{8}: pMtoC3 = plasticity rule for MEC to CA3 E weights
%       0: no plasticity
%       1: postsynaptic-gated rate-based plasticity
%   varargin{9}: t_updateWEtoE = time interval for updating WEtoE (ms)
%   varargin{10}: t_updateWMtoC3 = time interval for updating WMtoC3 (ms)
%   varargin{11}: newFlag: 1: generate new weight matrices and grid cells
%                          0: keep old weight matrices and grid cells
%   varargin{12}: coll: 1: use collateral EtoE connections
%                       0: no collateral connections
%   varargin{13}: wrFlag: 0: don't write to file
%                         1: write to file
%                         2: write to file, including histogram of WMtoC3 every 30 degrees during the first lap
%
%   Example: dname = hippomat2(1, 270, 600, 48, 1, 1, 0, 0, 50, 1, 1, 1,1)
%
%   To plot, use nethist.m (for weight matrix histograms and spikes),
%       nethistFormRate.m (for place field formation), or nethistBSRate.m (for backward shift)
%
%   To run with the gui: narargin = 6
%   dt is set in hippomat_gui2.m
%   varargin{1}: handles = handles to parameter information set in gui
%   varargin{2}: C3_E = struct containing parameter info for CA3 E cells
%   varargin{3}: C3_I = struct containing parameter info for CA3 I cells
%   varargin{4}: plotFlag
%       0: run the entire code
%       1: plot WMtoC3 and WEtoE in the gui and return
%       2: plot WEtoI and WItoE in the gui and return
%
%   Example: dname = hippomat(1, handles, C3_E, C3_I, 0)
%

function run_name = hippomat2grid(varargin)

persistent WEtoE0bar WMtoC30 WEtoI WItoE tilt base offset phi

% persistent variables: 
% WEtoE0bar: initial weight matrix for CA3 E to CA3 E.  If coll == 1, WEtoE0 = WEtoE0bar.  If coll == 0, WEtoE0 = 0*WEtoE0bar.
% WMtoC30: initial weight matrix for MEC to CA3 E.
% WEtoI, WItoE: constant weights for CA3 E to CA3 I and CA3 I to CA3 E.
% tilt, base, offset, phi: parameters defining each grid cell

% close all

%persistent variables only cleared when gui is first opened
%if not using the gui, the user may clear them from the command line

%%
if nargin == 13  % code run without the gui
    
    % Define inputs
    dt = varargin{1};
    Ne_M = varargin{2};
    Ne_C3 = varargin{3};
    Ni_C3 = varargin{4};
    SESSIONS = varargin{5};
    LAPS = varargin{6};
    pC3toC3 = varargin{7};
    pMtoC3 = varargin{8};
    t_updateWEtoE = varargin{9};
    t_updateWMtoC3 = varargin{10};
    newFlag = varargin{11};
    coll = varargin{12};
    wrFlag = varargin{13};
    
    % Initialize struct for CA3 E cells
    ee = zeros(Ne_C3,1);
    C3_E = struct(...
        'tauM', 20,...               % membrane voltage decay constant (ms)
        'Vrest', -70,...             % resting potential (mV)
        'Vreset', -65,...            % reset potential (mV)
        'Vth', -54,...               % threshold potential (mV)
        'tref', 3,...                % refractory period (ms)
        'gI', ee,...                 % inh. conductance (unitless)
        'gE', ee,...                 % exc. conductance (unitless)
        'tau_gE', 5,...              % exc. conductance decay constant (ms)
        'tau_gI', 5,...              % inh. conductance decay constant (ms)
        'gEsc1', 1,...               % constants to be used in while loop, TO BE SET
        'gEsc2', 1,...
        'gIsc1', 1,...
        'gIsc2',1,...
        'Ein', -70,...               % inh. reversal potential (mV)
        'Eex',0,...                  % exc. reversal potential (mV) THE CODE CURRENTLY ASSUMES THIS VALUE IS ZERO
        'v', ee,...                  % voltage (mV), TO BE INITIALIZED
        'refracCells', ee,...        % times at which each cell leaves its refractory period (ms)
        'quot', 1,...                % constant to be used in while loop, TO BE SET
        'spks', [],...               % cell struct for storing indeces of spiked cells, used for updating WEtoE, TO BE INITIALIZED
        'N', Ne_C3,...               % number of cells
        'last_spk', ee-1000,...      % vector storing time (ms) of most recent spike for each cell, used for updating WEtoE
        'pre_eff',[],...             % matrix for presynaptic efficacy of each cell, TO BE INITIALIZED
        'post_eff',[],...            % matrix for postsynaptic efficacy of each cell, TO BE INITIALIZED
        'pres',[],...                % cell struct storing indeces of CA3 E pre cells, TO BE SET
        'posts',[],...               % cell struct storing indeces of CA3 E post cells, TO BE SET
        'mecPres',[],...             % struct for storing indeces of MEC pre cells, TO BE SET
        'rate',ee);                  % rate of each cell (kHz)
    
    % Initialize struct for inh. CA3 cells
    ii = zeros(Ni_C3,1);
    C3_I = struct(...
        'tauM', 20,...               % membrane voltage decay constant (ms)
        'Vrest', -70,...             % resting potential (ms)
        'Vreset', -65,...            % reset potential (mV)
        'Vth', -54,...               % threshold potential (mV)
        'tref', 3,...                % refractory period (ms)
        'gE', ii,...                 % excitatory conductance of CA3 I cells
        'tau_gE', 5,...              % excitatory conductance decay constant (ms)
        'gEsc1', 1,...               % constants to be used in while loop, TO BE SET
        'gEsc2',1,...
        'Eex', 0,...                 % exc. reversal potential (mV); CODE ASSUMES VALUE IS 0
        'v', ii,...                  % voltage for each cell (mV), TO BE INITIALIZED
        'refracCells', ii,...        % time that each cell leaves its refractory period (ms)   
        'quot', 1,...                % constant to be used in while loop, TO BE SET
        'spks', [],...               % vector storing indeces of cells spiking at current timestep
        'N', Ni_C3);                 % number of cells
    
    % Initialize struct for MEC cells
    mm = zeros(Ne_M,1);
    mec = struct(...
        'tilt',mm,...           % tilt of grid (rad), TO BE SET
        'base',mm,...           % base of grid (cm), TO BE SET
        'offset',mm,...         % magnitude of grid offset (cm), TO BE SET
        'phi',mm,...            % angle of offset (rad), TO BE SET
        'gamma',.018,...        % determines spread of grids
        'disSq',[],...          % vector for the squared distance to the appropriate cells' grids (cm^2)
        'denom',mm,...          % constants used to update mec, TO BE SET
        'maxFreq',20*1e-3,...   % maximum freq for Poisson trains (kHz)
        'frec',0,...            % reciprocal of max frequency, TO BE SET
        'tref',3,...            % refractory period (ms)
        'ISIend',mm,...         % vector for ends of proposed ISIs (ms), TO BE INITIALIZED
        'spks', [],...          % cell vector storing indeces of cells spiked at current timestep
        'N', Ne_M,...           % number of MEC cells
        'rate',mm);             % rate of each cell (kHz)

    % Set struct for WMtoC3 plasticity
    pMtoC3S = struct(...
        'rule', pMtoC3, ...          % plasticity rule to use (0 for no rule, 1 for post-gated rate-based)
        'tUpdate',t_updateWMtoC3,... % time taken before updating WMtoC3 (ms)
        'dtUpdate',0,...             % steps taken before updating WMtoC3, TO BE SET
        'k', 25,...                  % constant for plasticity rate (ms)
        'thr',5e-3);                 % threshold rate for rate-based plasticity (kHz)
    
    % Set struct for WEtoE plsticity
    pC3toC3S = struct(...
        'rule', pC3toC3, ...         % rule to use (0 for no plasticity, 1 for Song and Abbott, 2 for Froemke and Dan)
        'Amin', 21, ...              % percentage of max weight for LTD S&A or F&D (was .53, Aplus was .5)
        'Aplus', 20, ...             % percentage of max weight for LTP S&A or F&D
        'tauMin', 20, ...            % decay rate for LTD (S&A or F&D) (ms)
        'tauPlus', 20, ...           % decay rate for LTP (S&A or F&D) (ms)
        'tauS_pre', 20, ...          % decay rate for pre efficacies (F&D) (ms)
        'tauS_post', 50,...          % decay rate for post efficacies (F&D) (ms)
        'tUpdate',t_updateWEtoE,...  % time taken until updating WEtoE (ms)
        'dtUpdate',0,....            % steps taken before updating WEtoE, TO BE SET
        'winsteps',0);               % steps in LTD or LTP window, TO BE SET
    
    % Set struct for computing instantaneous rates
    rateStr = struct(...
        'alpha',100,...   % calcium decay rate (ms); 100 ms taken by Francesco; usually about 20 ms
        'beta',10e-3,...  % rate constant (kHz)
        'sc',0);          % constant used to compute the rate, TO BE SET
    
    % Change newFlag if need be
    if newFlag == 0
        if isempty(WEtoE0bar) || isempty(tilt)
            newFlag=1;
            disp('set new flag to 1')
        end
    end
    
    % Set grid cell parameters
    if newFlag == 1

        % Set grid cell parameters
        tilt = pi/3*rand(mec.N,1);
        base = 22*rand(mec.N,1)+28;
        offset = 33*rand(mec.N,1);
        phi = 2*pi*rand(mec.N,1);

    end
    
    % Set max weights and densities for weight matrices  (multiplied
    % previous values by 5 to account for division by 5 due to tau error in
    % C3_E.gEsc2 and C3_E.gIsc2
    
    wMax_MtoC3 = 1.5;   rho_MtoC3 = 0.2;
    wMax_EtoE = 1.5;    rho_EtoE = 0.25;
    wMax_ItoE = 4;      rho_ItoE = 0.2;
    wMax_EtoI = 4;      rho_EtoI = 0.2;

    wInit_MtoC3 = .9;  %Initial values for synapses
    wInit_EtoE = .1;  
    
end  % initialize code without the gui

%%
if nargin == 8  % code run with gui
    handles = varargin{1};
    mec = varargin{2};
    gridParams = varargin{3};
    C3_E = varargin{4};
    C3_I = varargin{5};
    rateStr = varargin{6};
    pMtoC3S = varargin{7};
    pC3toC3S = varargin{8};
    
    wrFlag = 1;  %always write to file when using the gui
    
    % Set parameters from hippomat_gui2.fig
    dt = str2double(get(handles.dt,'String'));
    mec.N = str2double(get(handles.Ne_M,'String'));
    C3_E.N = str2double(get(handles.Ne_C3,'String'));
    C3_I.N = str2double(get(handles.Ni_C3,'String'));
    SESSIONS = str2double(get(handles.sessions,'String'));
    LAPS = str2double(get(handles.laps,'String'));
    rho_MtoC3 = str2double(get(handles.rho_MtoC3,'String'));
    rho_EtoE = str2double(get(handles.rho_EtoE,'String'));
    rho_EtoI = str2double(get(handles.rho_EtoI,'String'));
    rho_ItoE = str2double(get(handles.rho_ItoE,'String'));
    wMax_MtoC3 = str2double(get(handles.wMax_MtoC3,'String'));
    wMax_EtoE = str2double(get(handles.wMax_EtoE,'String'));
    wMax_EtoI = str2double(get(handles.wMax_EtoI,'String'));
    wMax_ItoE = str2double(get(handles.wMax_ItoE,'String'));
    wInit_MtoC3 = str2double(get(handles.wInit_MtoC3,'String'));
    wInit_EtoE = str2double(get(handles.wInit_EtoE,'String'));
    pMtoC3S.rule = get(handles.pMtoC3,'Value')-1;
    pC3toC3S.rule = get(handles.pC3toC3,'Value')-1;
    newFlag = get(handles.newFlag,'Value');
    
    if rho_EtoE == 0  %no collateral connections
        coll = 0;
    else
        coll = 1;  %collateral connections
    end
  
    % Change newFlag if need be
    if newFlag == 0
        if isempty(WEtoE0bar) || isempty(tilt)
            newFlag=1;
            disp('set new flag to 1')
        end
    end
    
    % Set grid cell parameters
    if newFlag
    
        % Convert to radians
        gridParams.tiltMin = gridParams.tiltMin * pi/180;
        gridParams.tiltMax = gridParams.tiltMax * pi/180;

        gridParams.phiMin = gridParams.phiMin * pi/180;
        gridParams.phiMax = gridParams.phiMax * pi/180;
        
        % Set persistent grid cell parameters
        range = gridParams.tiltMax - gridParams.tiltMin;
        tilt = range*rand(mec.N,1) + gridParams.tiltMin;
        
        range = gridParams.baseMax - gridParams.baseMin;
        base = range*rand(mec.N,1) + gridParams.baseMin;
        
        range = gridParams.offsetMax - gridParams.offsetMin;
        offset = range*rand(mec.N,1) + gridParams.offsetMin;

        range = gridParams.phiMax - gridParams.phiMin;
        phi = range*rand(mec.N,1) + gridParams.phiMin;
    
    end

    % Set remainder of mec struct
    mec.rate = zeros(mec.N,1);
    mec.maxFreq = mec.maxFreq*1e-3;  %kHz
    
    % Set remainder of C3_E struct
    ee = zeros(C3_E.N,1);
    C3_E.gI = ee;
    C3_E.gE = ee;
    C3_E.v  = ee;  %Add C3_E.Vrest later, both with or without the gui
    C3_E.refracCells = ee;
    C3_E.last_spk = -1000+ee;
    C3_E.rate = ee;
    
    % Set remainder of C3_I struct
    ii = zeros(C3_I.N,1);
    C3_I.gE = ii;
    C3_I.v = ii;  %will add C3_I.Vrest later, both with and w/o the gui
    C3_I.refracCells = ii;
    
    % Set remainder of pMtoC3S
    pMtoCS.thr = pMtoC3S.thr*1e-3;  %kHz
    
    % Set remainder of rateStr
    rateStr.beta = rateStr.beta*1e-3;  %kHz
    
end  % initialize code with the gui


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% End parameter setting
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%%

% Set persistent parameters
if newFlag
    
    % Set weight matrices
    WMtoC30 = wInit_MtoC3*spones(sprand(C3_E.N,mec.N,rho_MtoC3));  % initialize all values to the same small value
    WEtoE0bar = wInit_EtoE*spones(sprand(C3_E.N,C3_E.N,rho_EtoE));
    WEtoE0bar = WEtoE0bar - spdiags(diag(WEtoE0bar),0,C3_E.N,C3_E.N);  %delete diagonal synapses
    WItoE = wMax_ItoE*sprand(C3_E.N,C3_I.N,rho_ItoE);
    WEtoI = wMax_EtoI*sprand(C3_I.N,C3_E.N,rho_EtoI);
    disp('set new weights')

end

if coll == 0
    WEtoE0 = 0*WEtoE0bar;
else
    WEtoE0 = WEtoE0bar;
end

WMtoC3 = WMtoC30;
WEtoE = WEtoE0;

% Find the synapses used to compute the weight histograms
if wrFlag
    MtoC3syn = find(WMtoC30);
    EtoEsyn = find(WEtoE0);
end

% Store pre- and postsynaptic cells
for i=1:C3_E.N,
    C3_E.pres{i} = find(WEtoE0(i,:));
    C3_E.posts{i} = find(WEtoE0(:,i));
    C3_E.mecPres{i} = find(WMtoC30(i,:));
end

pMtoC3S.dtUpdate = ceil(pMtoC3S.tUpdate/dt);
pC3toC3S.dtUpdate = ceil(pC3toC3S.tUpdate/dt);

% Set space for the cell struct used for updating WEtoE
C3_E.spks{pC3toC3S.dtUpdate} = [];

mec.tilt = tilt;
mec.base = base;
mec.offset = offset;
mec.phi = phi;

mec.denom = mec.gamma*mec.base.^2;
mec.frec = 1/mec.maxFreq;

if newFlag == 1
    scgridvec2(mec,[],0,0);  %set persistent params in scgridvec2
end

rateStr.sc = rateStr.alpha / (rateStr.alpha + dt);

C3_E.gEsc1 = (C3_E.tau_gE-dt/2) / (C3_E.tau_gE + dt/2);
C3_E.gIsc1 = (C3_E.tau_gI-dt/2) / (C3_E.tau_gI + dt/2);
C3_I.gEsc1 = (C3_I.tau_gE-dt/2) / (C3_I.tau_gE + dt/2);

C3_E.gEsc2 = 1 / (C3_E.tau_gE + dt/2);
C3_E.gIsc2 = 1 / (C3_E.tau_gI + dt/2);
C3_I.gEsc2 = 1 / (C3_I.tau_gE + dt/2);

C3_E.quot = C3_E.tauM/dt;
C3_I.quot = C3_I.tauM/dt;

C3_E.v = C3_E.Vrest + C3_E.v;
C3_I.v = C3_I.Vrest + C3_I.v;

% Set the number of steps needed to look back from the curent position within pC3toC3S.dtUpdate for LTP or LTD
win_LTP = 50/dt;  % number of steps for LTP
win_LTD = 50/dt;  % number of steps for LTD
pC3toC3S.winsteps = max(win_LTP, win_LTD);

steps = pC3toC3S.winsteps+pC3toC3S.dtUpdate+1;  %number of steps needed to store efficacy values

if pC3toC3S.rule == 2
    appx = ceil(.25*C3_E.N*steps);
    C3_E.pre_eff = spalloc(C3_E.N,steps,appx);
    C3_E.post_eff = spalloc(C3_E.N,steps,appx);

    % Set persistent params in C3toC3_plasticity
    C3toC3_plasticity([], [], dt, pC3toC3S, C3_E, wMax_EtoE, []);
end

rad  =  33.0;                 % radius of the track (cm)
Laptime = 18*1e3;             % time taken to complete a lap (ms)
degTime = ceil(Laptime/360);  % time per degree (ms)

% Create a new directory for run storage
if wrFlag 
    run_num = size(dir('run*'),1)+1;
    run_name = ['run' num2str(run_num)];
    mkdir(run_name);           
end

wh = waitbar(0,'Session 1, % of laps completed');

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Initialization complete
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%%
for session_no=1:SESSIONS
    
    % Reset variables
    if session_no > 1 
        
        C3_E.v = 0*C3_E.v + C3_E.Vrest;
        C3_E.refracCells = 0*C3_E.refracCells;
        C3_E.gE = 0*C3_E.gE;
        C3_E.gI = 0*C3_E.gI;
        C3_E.last_spk = -1000 + 0*C3_E.last_spk;
        
        C3_I.gE = 0*C3_I.gE;
        C3_I.v = 0*C3_I.v + C3_I.Vrest;
        C3_I.refracCells = 0*C3_I.refracCells;
        
        % rotate and shift grid: mec.tilt, mec.offset, mec.phi
        
    end
        
    ISI = max(mec.tref, exprnd(mec.frec,mec.N,1));  %initial proposed ISIs (ms), mec.frec = 1/mec.maxFreq (ms)
    mec.ISIend = ISI;  %time proposed ISIs expire for each cell (ms)
        
    waitbar(0,wh,['Session ' num2str(session_no) ', % of laps completed']);
    
    simtime = 0;
    timestep = 0;
        
    if wrFlag
        
        % Open files to write weight histograms at each lap
        WMtoC3_dat = fopen([run_name '/WMtoC3-data-ses-' num2str(session_no)],'wb');
        WEtoE_dat = fopen([run_name '/WEtoE-data-ses-' num2str(session_no)],'wb');
        
        if wrFlag==2
            WMtoC3_dat1 = fopen([run_name '/WMtoC3-data1-ses-' num2str(session_no)],'wb');
        end
        
        cent_MtoC3 = linspace(0,wMax_MtoC3,20);
        cent_EtoE = linspace(0,wMax_EtoE,20);
        [numofW, Wvals] = hist(WMtoC3(MtoC3syn),cent_MtoC3);
        if any(size(numofW) ~= size(Wvals))
            Wvals = Wvals';
        end
        fwrite(WMtoC3_dat, [numofW' Wvals'], 'single');
        
        if coll
            [numofW, Wvals] = hist(WEtoE(EtoEsyn),cent_EtoE);
            fwrite(WEtoE_dat, [numofW' Wvals'], 'single');
        end
        
        % Write initial WMtoC3 to file
        WMtoC3_init = fopen([run_name '/WMtoC3-init-ses-' num2str(session_no)],'wt');
        for post=1:C3_E.N
            pres = C3_E.mecPres{post};
            for k = 1:length(pres)
                fprintf(WMtoC3_init,'%i %i %f\n',pres(k),post,WMtoC3(post,pres(k)));
            end
        end
        fclose(WMtoC3_init);

    end  %wrFlag
    
    flag = 0;
    
    for lap=1:LAPS
        
        disp(['lap number ' num2str(lap)])
        
        if wrFlag
            
            % Open files to write spiked cell  
            str = [num2str(session_no) '-lap-' num2str(lap)];
            mec_dat = fopen([run_name '/mec-data-ses-' str],'wb');
            ca3e_dat = fopen([run_name '/ca3e-data-ses-' str],'wb');
            ca3i_dat = fopen([run_name '/ca3i-data-ses-' str],'wb');
       
        end
        
        for iang = 1:360,

            if mod(iang,60) == 0
                waitbar((iang+(lap-1)*360)/(360*LAPS),wh)
            end
            
            % Write current angle
            if wrFlag
                fwrite(mec_dat,-iang,'int16');      
                fwrite(ca3e_dat,-iang,'int16');
                fwrite(ca3i_dat,-iang,'int16');
            end
            
            degt = 0;
            
            if wrFlag
                spiked_M_cumm = [];
                spikedIn_C_cumm = [];
                spikedEx_C_cumm = [];
            end
%%
            while degt < degTime    % 50 ms per degree
                
                degt = degt + dt;                        % amount of time spent at this degree (ms)
                ang = (degt/degTime + (iang-1))*pi/180;  % current angle (rad)
                simtime = simtime + dt;                  
                timestep = timestep + 1;
                
                mrk = mod(timestep-1,steps)+1;                  % marker for pre and post efficacies
                curr = mod((timestep-1),pC3toC3S.dtUpdate)+1;   % current place within updating WEtoE
                
                indChk = find(simtime > mec.ISIend);  %mec cells whose proposed ISI has expired
                
                
                % Update MEC and CA3 cells
                if ~isempty(indChk)
                    % Compute the rat's distance to the grid for mec cells with expired ISIs.
                    mec.disSq = scgridvec2(mec,ang,rad,indChk);
                
                    mec = kw_update_mec(mec,indChk,simtime, rateStr);
                else
                    mec.spks = [];
                end
               
                [C3_E, C3_I] = update_CA3(C3_E, C3_I, WMtoC3, WEtoE, WEtoI,...
                    WItoE, simtime, mec.spks, mrk, pC3toC3S, curr, rateStr);
                
                % Store spikes to be written to file every degree
                if wrFlag
                    spiked_M_cumm = [spiked_M_cumm; mec.spks];
                    spikedEx_C_cumm = [spikedEx_C_cumm; C3_E.spks{curr}];
                    spikedIn_C_cumm = [spikedIn_C_cumm; C3_I.spks];
                end
                
                % Update WMtoC3
                if pMtoC3S.rule && mod(timestep,pMtoC3S.dtUpdate)==0
                        
                    [WMtoC3, mec, C3_E] = MtoC3_plasticity(pMtoC3S, C3_E,...
                        WMtoC3, mec,wMax_MtoC3, timestep, rateStr);
                
                end
                
                % Update WEtoE
                if pC3toC3S.rule && curr == pC3toC3S.dtUpdate 
                    
                    WEtoE = C3toC3_plasticity(WEtoE, timestep, dt, ...
                        pC3toC3S, C3_E, wMax_EtoE, mrk);
                    
                end
                
%                 plot(simtime*ones(size(C3_E.spks{1})),C3_E.spks{1},'.')
%                 hold on
                
            end % while degt

%             pause
            
%%            
            % Write spikes to file
            if wrFlag
                
                fwrite(mec_dat,spiked_M_cumm,'int16');
                fwrite(ca3e_dat,spikedEx_C_cumm,'int16');
                fwrite(ca3i_dat,spikedIn_C_cumm,'int16');
            
                % Write weight histogram to file if in lap 1
                if wrFlag == 2 && lap == 1 && mod(iang,30) == 0
                    [numofW, Wvals] = hist(WMtoC3(MtoC3syn),cent_MtoC3);
                    fwrite(WMtoC3_dat1, [numofW' Wvals'], 'single');
                end
            
            end
            
%             if length(spikedEx_C_cumm) > C3_E.N*10
%                 flag = 1;
%             end
%             if flag == 1
% %                 fineplot(dt,simtime,C3_E,C3_I,mec)
%             end
            
        end  % for iang
        
        if wrFlag
            
            % Close files that just store data for one lap
            fclose(mec_dat);
            fclose(ca3e_dat);
            fclose(ca3i_dat);
            
            % Write weight histograms to file
            [numofW, Wvals] = hist(WMtoC3(MtoC3syn),cent_MtoC3);
            if any(size(numofW)~=size(Wvals))
                Wvals = Wvals';
            end
            fwrite(WMtoC3_dat, [numofW' Wvals'], 'single');
            
            if coll
                [numofW, Wvals] = hist(WEtoE(EtoEsyn),cent_EtoE);
                fwrite(WEtoE_dat, [numofW' Wvals'], 'single');
            end 
        
%             % Write actual WMtoC3 to file
%             WMtoC3_lap = fopen([run_name '/WMtoC3-lap' num2str(lap) 'ses-' num2str(session_no)],'wt');
%             for post=1:C3_E.N
%                 pres = C3_E.mecPres{post};
%                 for k = 1:length(pres)
%                     fprintf(WMtoC3_lap,'%i %i %f\n',pres(k),post,WMtoC3(post,pres(k)));
%                 end
%             end
%             fclose(WMtoC3_lap);

        end  %if wrFlag
        
    end % for laps
    
    if wrFlag
        % Write final WMtoC3 to file
        WMtoC3_fin = fopen([run_name '/WMtoC3-fin-ses-' num2str(session_no)],'wt');
        for post=1:C3_E.N
            pres = C3_E.mecPres{post};
            for k = 1:length(pres)
                fprintf(WMtoC3_fin,'%i %i %f\n',pres(k),post,WMtoC3(post,pres(k)));
            end
        end
        fclose(WMtoC3_fin);
    end %if wrFlag
    
end   % for sessions

if wrFlag
   
    fclose(WMtoC3_dat);
    fclose(WEtoE_dat);
    
    if wrFlag==2
        fclose(WMtoC3_dat1)
    end
 
end

close(wh)

return

%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Update CA3 exc. and inh. cells
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

function [C3_E, C3_I] = update_CA3(C3_E, C3_I, WMtoC3, WEtoE, WEtoI, ...
    WItoE, simtime, spikedM, mrk, pC3toC3S, curr, rateStr)

% Input:  C3_E, C3_I = structs for CA3 cells
%         WMtoC3, WEtoE, WEtoI, WItoE = weight matrices
%         simtime = current time of simulation (ms)
%         spikedM = indeces of MEC cells that just spiked
%         mrk = marker specifying location within efficacy matrices
%         pC3toC3S = struct for WEtoE plasticity
%         curr = marker specifying location within C3_E.spks
%         currL = marker specifying location within C3_E.spksL and mec.spksL
%
% Output: C3_E, C3_I = structs for CA3 cells


% Find currLast, the previous value of curr.  Cells that spiked at currLast now affect conductances
if curr == 1
    currLast = pC3toC3S.dtUpdate;
else
    currLast = curr - 1;
end

% Update conductances, see hippomat2grid.pdf for equation derivations
C3_E.gE = C3_E.gEsc1*C3_E.gE + C3_E.gEsc2*(sum(WMtoC3(:,spikedM),2) + ...
    sum(WEtoE(:,C3_E.spks{currLast}),2));

C3_E.gI = C3_E.gIsc1*C3_E.gI + C3_E.gIsc2*sum(WItoE(:,C3_I.spks),2);

C3_I.gE = C3_I.gEsc1*C3_I.gE + C3_I.gEsc2*sum(WEtoI(:,C3_E.spks{currLast}),2);

% Update voltages, see hippomat2grid.pdf for equation derivation
vtop_E = C3_E.quot*C3_E.v + C3_E.Vrest + C3_E.gI*C3_E.Ein;
vbot_E = C3_E.quot + 1 + C3_E.gE + C3_E.gI;

vtop_I = C3_I.quot*C3_I.v + C3_I.Vrest;
vbot_I = C3_I.quot + 1 + C3_I.gE;

C3_E.v = vtop_E ./ vbot_E;
C3_I.v = vtop_I ./ vbot_I;

% Reset refractory cells
refracIndex_E = find(C3_E.refracCells > simtime);
refracIndex_I = find(C3_I.refracCells > simtime);

C3_E.v(refracIndex_E) = C3_E.Vreset;
C3_I.v(refracIndex_I) = C3_I.Vreset;

% Find spiked cells
spkdEx = find(C3_E.v > C3_E.Vth);
spkdIn = find(C3_I.v > C3_I.Vth);

% Store spiked cells
C3_E.spks{curr} = spkdEx;  
C3_I.spks = spkdIn;

% Reset voltage of spiked cells
C3_E.v(spkdEx) = C3_E.Vreset;
C3_I.v(spkdIn) = C3_I.Vreset;

% Set refractory times for spiked cells
C3_E.refracCells(spkdEx) = simtime + C3_E.tref;
C3_I.refracCells(spkdIn) = simtime + C3_I.tref;

% Update rate
C3_E.rate = rateStr.sc * C3_E.rate;
C3_E.rate(spkdEx) = C3_E.rate(spkdEx) + rateStr.beta;

% Store efficacies for each spike
if pC3toC3S.rule == 2  %Froemke and Dan
    
    C3_E.pre_eff(:,mrk) = 0;  % Clear any efficacy values at the current mrk.
    C3_E.post_eff(:,mrk) = 0;
    deltat_eff = simtime - C3_E.last_spk(spkdEx);  % time since the previous spike of each spiked cell
    C3_E.pre_eff(spkdEx, mrk) = 1 - exp(-(deltat_eff/pC3toC3S.tauS_pre));
    C3_E.post_eff(spkdEx, mrk) = 1 - exp(-(deltat_eff/pC3toC3S.tauS_post));
    C3_E.last_spk(spkdEx) = simtime;

end

% Note: if pC3toC3S.rule == 1, then I'll update C3_E.last_spk in
% C3toC3_plasticity

return

%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Change WMtoC3 according to the plasticity rule pMtoC3
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

function [WMtoC3, mec, C3_E] = MtoC3_plasticity(pMtoC3S, C3_E, WMtoC3, mec,...
    wMax_MtoC3, timestep, rateStr)

% Input: pMtoC3S = struct for WMtoC3 plasticity
%        C3_E, mec = structs for CA3 E and MEC cells
%        WMtoC3 = weights from MEC cells to CA3 E cells
%        wMax_MtoC3 = max weight
%        currL = current position in spksL matrices for computing rate
%        timestep = current timestep
%        rateStr = struct for computing rate
%
% Output: updated WMtoC3
%         mec (changed mec.rate)
%         C3_E (changed C3_E.rate)

% Postsynaptic rate-based plasticity, used in Francesco's code
if pMtoC3S.rule == 1 
    
    tol = 1e-6;  %kHz
    posts = find(C3_E.rate > tol);

    for k=1:length(posts)
        post = posts(k);
        pres = C3_E.mecPres{post};
        
        dW = pMtoC3S.tUpdate * pMtoC3S.k * (mec.rate(pres)-pMtoC3S.thr) * C3_E.rate(post);  % ms^2 * kHz^2 = unitless
        WMtoC3(post,pres) = min(max(WMtoC3(post,pres) + dW', 0), wMax_MtoC3);      
    end
    
end

return

%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Change WEtoE according to pC3toC3
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

function WEtoE = C3toC3_plasticity(WEtoE, timestep, dt, pC3toC3S, C3_E,...
    wMax_EtoE, mrk)

% Input: WEtoE = weights from CA3 E cells to CA3 E cells;
%           set to empty if want to set persistant params F_LTP and F_LTD
%           and return
%        timestep = current timestep
%        dt = time for each timestep (ms)
%        pC3toC3S = struct for WEtoE plasticity
%        C3_E = struct for CA3 E cells
%        wMax_EtoE = max weight
%        mrk = current position within efficacy matrices
%
% Output: updated WEtoE

%pC3toC3S.winsteps = number of steps to look back for LTP/LTD
%pC3toC3S.dtUpdate = number of steps taken until updating WEtoE

persistent F_LTP F_LTD

% Constants needed for the STDP F-curve
tauPlus = pC3toC3S.tauPlus;
tauMin = pC3toC3S.tauMin;
Ap = .01*pC3toC3S.Aplus*wMax_EtoE;
Am = .01*pC3toC3S.Amin*wMax_EtoE;

% Store weight changes and indeces according to all spikes in the last dtUpdateW steps

%%
if pC3toC3S.rule == 1

    % implementation Song, Miller, and Abbott method.  I'm only taking into account the last spike here, not
    % all previous spikes

    % Set up storage for weight change over last pC3toC3S.dtUpdate steps
    appx = ceil(pC3toC3S.dtUpdate*.03*.15*C3_E.N^2);  %(timesteps)(3% spiking)(15% conn)
    entTemp = spalloc(appx,1,appx);  % indeces in WEtoE for each weight change
    dWTemp = spalloc(appx,1,appx);   % weight changes
    cnt = 1;

    tol = .001*wMax_EtoE;  %only count weight changes greater than tol
    
    % Look back through timesteps in dtUpdate
    for k=1:pC3toC3S.dtUpdate

        % Find the appropriate timestep that was passed in the last dtUpdate steps
        currStep = timestep - pC3toC3S.dtUpdate + k;  %(timestep = multiple of dtUpdate)
        currTime = currStep * dt;  %simtime at currStep (ms)

        % Find cells that spiked at currStep
        currSpks = C3_E.spks{k};    %C3_E.spks is a cell struct storing indeces of spiked cells; length = dtUpdate

        % Compute weight change due to each spike at currTime
        for ispk = 1:length(currSpks);

            % Grab cell that spiked, its pres, and its posts
            currSpk = currSpks(ispk);
            pres = C3_E.pres{currSpk};
            posts = C3_E.posts{currSpk};

            % LTP (consider when currSpk = postsynaptic)
            F = exp(-(currTime-C3_E.last_spk(pres)) / tauPlus);  %1 x length(pres)
            dWLTP = Ap * F;  %1 x length(pres)

            % Find indeces where weight change is greater than tol
            indBig = find(dWLTP >= tol);          
            
            % Convert synapse indeces (post,pre) to column major format
            convInd = (pres(indBig)-1)*C3_E.N+currSpk;  %(col-1)*Nrows+row

            % Store weight changes
            num = length(convInd);
            entTemp(cnt:cnt+num-1) = convInd;
            dWTemp(cnt:cnt+num-1) = dWLTP(indBig);
            cnt = cnt+num;

            % LTD (consider currSpk = presynaptic)
            F = exp(-(currTime-C3_E.last_spk(posts)) / tauMin);  %1 x length(posts)
            dWLTD = Am * F;  %1 x length(posts)

            % Find indeces where weight change is greater than tol
            indBig = find(dWLTD >= tol); 
            
            % Convert synapse indeces (post,pre) to column major format
            convInd = (currSpk-1)*C3_E.N+posts(indBig);   %(col-1)*Nrows+row

            % Store weight changes
            num = length(convInd);
            entTemp(cnt:cnt+num-1) = convInd;
            dWTemp(cnt:cnt+num-1) = -dWLTD(indBig);
            cnt = cnt+num;

        end   %ispks = 1:length(currSpks)

        % Update last spike information
        C3_E.lastSpk(currSpks) = currTime;  %ms
        
    end  %k=1:pC3toC3S.dtUpdate

%%
elseif pC3toC3S.rule == 2

    % Implementation of Froemke and Dan method (2002)

    steps = pC3toC3S.winsteps + pC3toC3S.dtUpdate + 1;  % number of columns in efficacy matrices

    % Set persistent values
    if isempty(WEtoE)

        F_LTP = zeros(steps,1);
        F_LTD = F_LTP;

        deltat = (pC3toC3S.winsteps:-1:0)*dt;   %(ms)
        F_LTP(end-pC3toC3S.winsteps:end) = Ap*exp(-deltat/tauPlus)';
        F_LTP(end) = 0;  %no weight increase for delta_t = 0
        F_LTD(end-pC3toC3S.winsteps:end) = Am*exp(-deltat/tauMin)';
        disp('set F')

        WEtoE = [];  %This value is not assigned to a variable when used here
        return
        
    end %persistent

    % Set up storage for weight change
    appx = ceil(pC3toC3S.dtUpdate*.03*.15*C3_E.N^2);  %(timesteps)(3% spiking)(15% conn)
    entTemp = spalloc(appx,1,appx);  % indeces in WEtoE for each weight change
    dWTemp = spalloc(appx,1,appx);   % weight changes
    cnt = 1;

    % Compute weight changes

    if timestep <= steps   % no shifting needed b/c haven't cycled through efficacy matrices yet

        % Look back through timesteps in dtUpdate
        for k=1:pC3toC3S.dtUpdate

            % Find the appropriate timestep that was passed in the last dtUpdate steps
            currStep = timestep - pC3toC3S.dtUpdate + k;  %timestep = multiple of dtUpdate

            % Find indeces needed to truncate the F vectors and efficacy matrices
            indF = steps-currStep+1 : steps;   %last currStep entries
            indEff = 1:currStep;               %first currStep entries

            % Find cells that spiked at currStep
            currSpks = C3_E.spks{k};    %C3_E.spks is a cell struct storing indeces of spiked cells; length = dtUpdate

            % Compute weight change due to each spike
            for ispk = 1:length(currSpks);

                % Grab cell that spiked, its pres, and its posts
                currSpk = currSpks(ispk);
                pres = C3_E.pres{currSpk};
                posts = C3_E.posts{currSpk};

                % LTP (consider when currSpk = postsynaptic)
                dWLTP = C3_E.pre_eff(pres,indEff) * F_LTP(indF) * ...
                    C3_E.post_eff(currSpk,currStep);    % dW = pre_eff*post_eff*F(tpre-tpost)
                convInd = (pres-1)*C3_E.N+currSpk;  %col major format

                % Store weight changes
                num = length(convInd);
                entTemp(cnt:cnt+num-1) = convInd;
                dWTemp(cnt:cnt+num-1) = dWLTP;
                cnt = cnt+num;

                % LTD (consider currSpk = presynaptic)
                dWLTD = C3_E.post_eff(posts,indEff) * F_LTD(indF) * ...
                    C3_E.pre_eff(currSpk,currStep);
                convInd = (currSpk-1)*C3_E.N+posts;   %column major format

                % Store weight changes
                num = length(convInd);
                entTemp(cnt:cnt+num-1) = convInd;
                dWTemp(cnt:cnt+num-1) = -dWLTD;
                cnt = cnt+num;

            end   %ispks = 1:length(currSpks)

        end  %k=1:pC3toC3S.dtUpdate

    else   %timestep > steps  (need to shift)

        % Loop through timesteps in dtUpdate
        for k=1:pC3toC3S.dtUpdate

            % Find column of the eff matrices corresponding to the appropriate timestep that was passed within the last dtUpdate steps
            currMrk = mrk - pC3toC3S.dtUpdate + k;
            if currMrk <= 0
                currMrk = steps+currMrk;  %mrk cycles between 1 and steps
            end

            % Shift F_LTP and F_LTD currMrk times (shifts down).
            Fshf_LTP = circshift(F_LTP,currMrk);
            Fshf_LTD = circshift(F_LTD,currMrk);

            % Find cells that spike at step currMrk
            currSpks = C3_E.spks{k};

            % Find weight change due to each spike at step currMrk
            for ispk = 1:length(currSpks);

                % Find pre- and postsynaptic cells to the spiked cell
                currSpk = currSpks(ispk);
                pres = C3_E.pres{currSpk};
                posts = C3_E.posts{currSpk};

                %LTP (consider currSpk = postsynaptic)
                dWLTP = C3_E.pre_eff(pres,:) * Fshf_LTP * ...
                    C3_E.post_eff(currSpk,currMrk);
                convInd = (pres-1)*C3_E.N+currSpk;  %col. major format

                % Store weight changes
                num = length(convInd);
                entTemp(cnt:cnt+num-1) = convInd;
                dWTemp(cnt:cnt+num-1) = dWLTP;
                cnt = cnt+num;

                %LTD (consider currSpk = presynaptic)
                dWLTD = C3_E.post_eff(posts,:) * Fshf_LTD * ...
                    C3_E.pre_eff(currSpk,currMrk);
                convInd = (currSpk-1)*C3_E.N+posts;  %col. major format

                % Store weight changes
                num = length(convInd);
                entTemp(cnt:cnt+num-1) = convInd;
                dWTemp(cnt:cnt+num-1) = -dWLTD;
                cnt = cnt+num;

            end  %ispk = 1:NeSpks

        end  %k=1:pC3toC3S.dtUpdate

    end  %else (timestep>steps)

end  %if pC3toC3S.rule

%%
% Change WEtoE according to weight changes and indeces stored

% Truncate to eliminate extra space never used
numTemp = cnt-1;
entTemp = entTemp(1:numTemp);
dWTemp = dWTemp(1:numTemp);

% Store no more than one weight change per synapse.

% Sort entTemp and dWTemp so that the matrix indeces are ascending
[entTemp,sortInd] = sort(entTemp);
dWTemp = dWTemp(sortInd);

% Preallocate space for the final indeces and weight changes (no more than one entry per synapse)
entFinal = 0*entTemp;
dWFinal = entFinal;

ptrF = 0; %index for entFinal and dWFinal
ptrT = 1; %index for entTemp and dWTemp

% Sum the weight changes for each synapse
while ptrT < numTemp

    dW = 0;

    ent = entTemp(ptrT);
    ptrF = ptrF+1;

    % Loop over all weight changes corresponding to the synapse ent
    while entTemp(ptrT) == ent && ptrT < numTemp
        dW = dW + dWTemp(ptrT);
        ptrT = ptrT+1;
    end

    entFinal(ptrF) = ent;
    dWFinal(ptrF) = dW;

end

% Store the last weight change in dWTemp
if numTemp

    if entTemp(numTemp) == ent %entTemp(numTemp-1) == entTemp(numTemp); add to last entry in dWFinal
        dWFinal(ptrF) = dWFinal(ptrF)+dWTemp(numTemp);
    else                       %entTemp(numTemp-1) ~= entTemp(numTemp); store a new synapse in entFinal/dWFinal
        ptrF = ptrF+1;
        entFinal(ptrF) = entTemp(numTemp);
        dWFinal(ptrF) = dWTemp(numTemp);
    end

end

% Truncate entFinal/dWFinal to eliminate unused space
entFinal = entFinal(1:ptrF);
dWFinal = dWFinal(1:ptrF);

% Update weight matrix according to all weight changes in the last dtUpdate steps
WEtoE(entFinal) = min(max(WEtoE(entFinal)+dWFinal, 0), wMax_EtoE);

return

%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Function: Calculate the rat's distance to the grid
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

function disSq = scgridvec2(mec,rang,rad,ind)
%
% Input: mec: mec struct
%        rang = angle of rat's position (rad); set to an empty matrix if
%           wanting to set persistent values and return
%        rad = radius of track (cm)
%        ind = index of cells whose distances are being computed
%
% Output: disSq: col vec of squared distances from the rat to the nearest 
% grid point for each grid cell
%

persistent h c m mSq c1 c2 c3 c4

if isempty(rang)
    N = length(mec.tilt);

    h = (mec.base/2)*tan(pi/3);  %grid height

    % calculate grid center
    c = zeros(N,2);
    ind1 = find(mec.phi<pi);
    c(ind1,1) = mec.offset(ind1).*cos(mec.phi(ind1));
    c(ind1,2) = mec.offset(ind1).*sin(mec.phi(ind1));

    ind2 = setdiff(1:N,ind1);
    eta = mec.phi(ind2)-pi;
    c(ind2,1) = -mec.offset(ind2).*cos(eta);
    c(ind2,2) = -mec.offset(ind2).*sin(eta);

    m = tan(mec.tilt);  % slope of grid latitudes
    mSq = m.^2;
    
    c1 = h.*-sin(mec.tilt);
    c2 = h.*cos(mec.tilt);
    c3 = (mec.base/2).*cos(mec.tilt);
    c4 = (mec.base/2).*sin(mec.tilt);

    disp('set new persistent vars in scgridvec2')
    
    return
end

N = length(ind);

% Calculate rat's position
rx = rad*cos(rang);
ry = rad*sin(rang);

% Calculate projection of rat's position onto meridian
p1 = (mSq(ind)*rx - m(ind)*ry + m(ind).*c(ind,2) + c(ind,1)) ./ (mSq(ind)+1);
p2 = c(ind,2) - (p1-c(ind,1))./m(ind);

% Step 2 in algorithm
ell1 = sqrt((p1-c(ind,1)).^2+(p2-c(ind,2)).^2);
n1 = fix(ell1./h(ind));

ell2 = sqrt((p1-rx).^2+(p2-ry).^2);     
n2 = fix(ell2./(mec.base(ind)/2));

% Box in rat's position: parallel to grid latitudes
qlo = zeros(N,2);
qlo(:,1) = c(ind,1) + sign(p2-c(ind,2)).*n1.*c1(ind);
qlo(:,2) = c(ind,2) + sign(p2-c(ind,2)).*n1.*c2(ind);

qhi = qlo;
qhi(:,1) = c(ind,1) + sign(p2-c(ind,2)).*(n1+1).*c1(ind);
qhi(:,2) = c(ind,2) + sign(p2-c(ind,2)).*(n1+1).*c2(ind);

% Box in rat's position: parallel to grid meridian
wlo = qlo;
wlo(:,1) = p1 + sign(rx-p1).*n2.*c3(ind);
wlo(:,2) = p2 + sign(rx-p1).*n2.*c4(ind);

whi = wlo;
whi(:,1) = p1 + sign(rx-p1).*(n2+1).*c3(ind);
whi(:,2) = p2 + sign(rx-p1).*(n2+1).*c4(ind);

% find true vertices
rem = mod(n1+n2,2);
odd = find(rem);
even = find(rem==0);

indOdd = ind(odd);
indEven = ind(even);

g1 = zeros(N,2);
g2 = zeros(N,2);

mOdd = m(indOdd);
mSqOdd = mSq(indOdd);
mEven = m(indEven);
mSqEven = mSq(indEven);

if ~isempty(odd)
    g1(odd,1) = (qlo(odd,1).*mSqOdd + (whi(odd,2)-qlo(odd,2)).*mOdd + whi(odd,1)) ./ (mSqOdd+1);
    g1(odd,2) = qlo(odd,2)+(g1(odd,1)-qlo(odd,1)).*mOdd;
    g2(odd,1) = (qhi(odd,1).*mSqOdd + (wlo(odd,2)-qhi(odd,2)).*mOdd + wlo(odd,1)) ./ (mSqOdd+1);
    g2(odd,2) = qhi(odd,2) + (g2(odd,1)-qhi(odd,1)).*mOdd;
end

if ~isempty(even)
    g1(even,1) = (qlo(even,1).*mSqEven + (wlo(even,2)-qlo(even,2)).*mEven + wlo(even,1)) ./ (mSqEven+1);
    g1(even,2) = qlo(even,2)+(g1(even,1)-qlo(even,1)).*mEven;
    g2(even,1) = (qhi(even,1).*mSqEven + (whi(even,2)-qhi(even,2)).*mEven + whi(even,1)) ./ (mSqEven+1);
    g2(even,2) = qhi(even,2)+(g2(even,1)-qhi(even,1)).*mEven;
end

disSq = min((rx-[g1(:,1),g2(:,1)]).^2+(ry-[g1(:,2),g2(:,2)]).^2, [], 2);

%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Update mec cells
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

function mec = kw_update_mec(mec,indChk,simtime, rateStr)

% Input: mec = mec struct
%        indChk = cells with expiried proposed ISIs
%        simtime = current time of the simulation (ms)
%        currL = marker specifying position in mec.spksL
%
% Output: mec = mec struct

% Compute the probability of spiking for indChk cells
P = exp(-mec.disSq./mec.denom(indChk));  %mec.denom = mec.gamma*mec.base.^2, mec.disSq = squared distance between rat and grid for indChk cells
r = rand;

% Find cells that spike
indSpk = find(P > r);
spkd = indChk(indSpk);

% Store spikes
mec.spks = spkd;

% Propose new ISIs for indChk cells
ISI = max(mec.tref, exprnd(mec.frec,length(indChk),1));
mec.ISIend(indChk) = simtime + ISI;

% Update rate
mec.rate = rateStr.sc * mec.rate;
mec.rate(spkd) = mec.rate(spkd) + rateStr.beta;

return

%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Plot conductances and volages for CA3 cells
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

function fineplot(dt,t,C3_E,C3_I,mec)

figure(4)      %  examine potentials of 1st 9 E and I cells
for sp=1:9,
    subplot(3,3,sp)
    plot(t,C3_E.v(sp),'.b')
    hold on
%     plot(t,C3_I.v(sp),'r.')
end
title(['C3E.v (blue) and C3I.v (red), dt = ' num2str(dt)])

figure(5)      %  examine conductances of 1st 9 E and I cells
for sp=1:9,
    subplot(3,3,sp)
    plot(t,C3_E.gE(sp),'.b')
    hold on
%     plot(t,C3_E.gI(sp),'r.')
%     plot(t,C3_I.gE(sp),'k.')
end
title(['C3E.gE (blue), C3E.gI (red) and C3I.gE (black), dt = ' num2str(dt)])

figure(6)
for sp=1:9
    subplot(3,3,sp)
    plot(t,C3_E.gI(sp),'.b')
    hold on
end
title('C3_E.gI')

return