%
% nethist(dname,sess,laps,Ne_M,Ne_C3,Ni_C3,coll,handles)
%
% Can be used to view output from hippomat2.m without the gui.
% Construct network histograms from output in c:\hippo\runX
%
%   dname: name of directory created in hippomat2.m
%   sess: total number of sessions
%   laps: total number of laps per session
%   Ne_M: number of MEC cells
%   Ne_C3: number of CA3 excitatory cells
%   Ni_C3: number of CA3 inhibitory cells
%   coll: 1 if have EtoE connections; 0 otherwise
%   handles: if running without the gui, set handles = []
%            if running with the gui, send in handles
%   plotFlag: use if running with the gui;
%             1: plot spike rate for MEC cells
%             2: plot spike rate for CA3 E cells
%             3: plot WMtoC3 hist
%             4: plot WEtoE hist
%

function nethist(dname0,sess,laps,Ne_M,Ne_C3,Ni_C3,coll,handles,plotFlag)

% I don't always know the directory name from the gui, so I must specify
% the directory name once, then keep it as a persistent variable.

persistent dname % make dname persistent b/c I don't always know dname when I use the gui

if ~isempty(dname0)
    dname = dname0;
end

if isempty(sess)  %I initially call nethist from the gui just to set dname
    return
end

cd(dname)

if isempty(handles) %run without the gui
    guiFlag = 0;
else                %run with the gui
    guiFlag = 1;
end 

fig0 = 0;

lap1plot = 0;  %1: plot WMtoC3 hist for first lap; 0 otherwise
if guiFlag == 1
    lap1plot = 0;  %never plotting first lap hist in gui
end

% Set up variables used to compute the spike rates
beta = 10e-3;  %kHz, should be the same as used in hippomat2grid.m
alpha = 100;  %ms, should be the same as used in hippomat2grid.m

dspkVec = 50*(14:-1:0)';  %ms
expVec = exp(-dspkVec/alpha);

MECrateT = zeros(Ne_M,360);       %total rate each session for MEC cells
MECrate = MECrateT;               %spike rate each lap for MEC cells

CA3rateT = zeros(Ne_C3,360);      %total rate each session for CA3 Exc. cells
CA3rate = CA3rateT;               %spike rate each lap for CA3 Exc. cells

mechistmean = zeros(Ne_M,360);  %average number of spikes per degree for each cell
ca3eSum = zeros(Ne_C3,360);  %total number of spikes per degree during one lap for each cell; resets for each lap
ca3iSum = zeros(Ni_C3,360);

for s=1:sess

    % Open file storing weight histograms for each lap for WMtoC3
    fname = ['WMtoC3-data-ses-' num2str(s)];
    aW = fopen(fname,'r');
    Wdat1 = fread(aW, 'single');
    fclose(aW);

    if guiFlag == 0
        figure(fig0+1)  %figure for weight histograms for WMtoC3
        plot3(Wdat1(21:40),zeros(20,1),Wdat1(1:20)/sum(Wdat1(1:20)))
        hold on
    elseif plotFlag == 3
        axes(handles.axes_main)
        plot3(Wdat1(21:40),zeros(20,1),Wdat1(1:20)/sum(Wdat1(1:20)))
        hold on
    end

    if lap1plot
        figure(fig0+3) %figure for weight histogram of WMtoC3 during first lap
        plot3(Wdat1(21:40),zeros(20,1),Wdat1(1:20)/sum(Wdat1(1:20)))
        hold on
    end

    % Open file storing weight histograms for each lap for WEtoE
    if coll

        fname = ['WEtoE-data-ses-' num2str(s)];
        aW = fopen(fname,'r');
        Wdat2 = fread(aW, 'single');
        fclose(aW);

        if guiFlag == 0
            figure(fig0+2) %figure for weight histograms for WEtoE
            plot3(Wdat2(21:40),zeros(20,1),Wdat2(1:20)/sum(Wdat2(1:20)))
            hold on
        elseif plotFlag == 4
            axes(handles.axes_main)
            plot3(Wdat2(21:40),zeros(20,1),Wdat2(1:20)/sum(Wdat2(1:20)))
            hold on
        end

    end

    % Plot histogram of WMtoC3 during the first lap
    if lap1plot
        fname = ['WMtoC3-data1-ses-' num2str(s)];
        aW = fopen(fname,'r');
        WdatLap1 = fread(aW, 'single');
        %size(Wdat)
        fclose(aW);

        figure(fig0+3)
        for d = 0:11

            st = d*40+1;
            z = WdatLap1(st:st+19);
            deg = (d+1)*30;

            %             if deg <= 120 || mod(deg,120)==0
            plot3(WdatLap1(st+20:st+39),deg*ones(20,1),z/sum(z))
            %             end

        end
        hold off
    end  % if lap1plot

    for ell=1:laps

        st = ell*40+1;

        % Plot the WMtoC3 histogram for lap ell
        if guiFlag == 0
            figure(fig0+1)
            z = Wdat1(st:st+19);
            plot3(Wdat1(st+20:st+39),ell*ones(20,1),z/sum(z))
        elseif plotFlag == 3
            axes(handles.axes_main)
            z = Wdat1(st:st+19);
            plot3(Wdat1(st+20:st+39),ell*ones(20,1),z/sum(z))
        end
        
        % Plot the WEtoE histogram for lap ell
        if coll
            if guiFlag == 0
                figure(fig0+2)
                z = Wdat2(st:st+19);
                plot3(Wdat2(st+20:st+39),ell*ones(20,1),z/sum(z))
            elseif plotFlag == 4
                axes(handles.axes_main)
                z = Wdat2(st:st+19);
                plot3(Wdat2(st+20:st+39),ell*ones(20,1),z/sum(z))
            end
            
        end

        if guiFlag == 0 || plotFlag == 1 || plotFlag == 2

            % Open file storing MEC spikes for lap ell
            str = [num2str(s) '-lap-' num2str(ell)];
            strm = ['mec-data-ses-' str];
            a = fopen(strm,'r');
            dat = fread(a,'int16');
            fclose(a);

            % Determine number of spikes per degree for each cell
            mechist = ghist(dat,Ne_M); %mechist = Ne_M x 360
            mechistmean = mechistmean + mechist;

            % Open file storing CA3E spikes for lap ell
            stre = ['ca3e-data-ses-' str];
            a = fopen(stre,'r');
            dat = fread(a,'int16');
            fclose(a);

            % Determine number of spikes per degree for each cell
            ca3ehist = ghist(dat,Ne_C3); %ca3ehist = Ne_C3 x 360
            ca3eSum = ca3eSum + ca3ehist;

            % Open file storing CA3I spikes for lap ell
            stri = ['ca3i-data-ses-' str];
            a = fopen(stri,'r');
            dat = fread(a,'int16');
            fclose(a);

            % Determine number of spikes per degree for each cell
            ca3ihist = ghist(dat,Ni_C3);
            ca3iSum = ca3iSum + ca3ihist;

            % Compute the spike rate for each MEC cell during lap ell
            MECrate = 0*MECrate;  %spike rate for each lap
            for deg = 1:360
                if deg <= 14
                    if ell == 1
                        MECrate(:,deg) = beta*mechist(1:Ne_M,1:deg)*expVec(end-deg+1:end);
                    else
                        MECrate(:,deg) = beta*mechist(1:Ne_M,[(346+deg):360,1:deg])*expVec;
                    end
                else
                    MECrate(:,deg) = beta*mechist(1:Ne_M,(deg-14):deg)*expVec;
                end
            end  %for deg

            MECrateT = MECrateT + MECrate;

            % Compute the spike rate for CA3 Exc. cells during lap ell
            CA3rate = 0*CA3rate;  %spike rate for each lap
            for deg = 1:360
                if deg <= 14
                    if ell == 1
                        CA3rate(:,deg) = beta*ca3ehist(1:Ne_C3,1:deg)*expVec(end-deg+1:end);
                    else
                        CA3rate(:,deg) = beta*ca3ehist(1:Ne_C3,[(346+deg):360,1:deg])*expVec;
                    end
                else
                    CA3rate(:,deg) = beta*ca3ehist(1:Ne_C3,(deg-14):deg)*expVec;
                end
            end  %for deg

            CA3rateT = CA3rateT + CA3rate;

        end  %if guiFlag==0 || plotFlag==1

    end  %for laps

    figs = 3*s;  %number of figures used so far

    % Plot MEC average spike rate
    if guiFlag == 0
        figure(fig0+figs+1)
        MECrateAve = MECrateT/laps;  %Ne_M x 360
        imagesc(MECrateAve'*1e3)
        xlabel('MEC Cells','fontsize',16)
        ylabel('Angle (deg)','fontsize',16)
        title('Average Spike Rate (Hz): MEC','fontsize',16)
        colorbar
    elseif plotFlag == 1
        axes(handles.axes_main)
        MECrateAve = MECrateT/laps;  %Ne_M x 360
        imagesc(MECrateAve'*1e3)
        xlabel('MEC Cells','fontsize',16)
        ylabel('Angle (deg)','fontsize',16)
        title('Average Spike Rate (Hz): MEC','fontsize',16)
        colorbar
    end

    % Plot CA3 Exc. average spike rate
    if guiFlag == 0
        figure(fig0+figs+2)
        CA3rateAve = CA3rateT/laps;  %Ne_C3 x 360
        imagesc(CA3rateAve'*1e3)
        xlabel('CA3 Exc. Cells','fontsize',16)
        ylabel('Angle (deg)','fontsize',16)
        title('Average Spike Rate (Hz): CA3 Exc.','fontsize',16)
        colorbar
    elseif plotFlag == 2
        axes(handles.axes_main)
        CA3rateAve = CA3rateT/laps;  %Ne_C3 x 360
        imagesc(CA3rateAve'*1e3)
        xlabel('CA3 Exc. Cells','fontsize',16)
        ylabel('Angle (deg)','fontsize',16)
        title('Average Spike Rate (Hz): CA3 Exc.','fontsize',16)
        colorbar
    end

    % Label WMtoC3 figure
    if guiFlag == 0
        figure(fig0+1)
        xlabel('Weight','fontsize',16)
        ylabel('Lap','fontsize',16)
        zlabel('Fraction','fontsize',16)
        title(['Evolution of WMtoC3: Session ' num2str(s) ''],'fontsize',16)
        hold off
    elseif plotFlag == 3
        axes(handles.axes_main)
        xlabel('Weight','fontsize',16)
        ylabel('Lap','fontsize',16)
        zlabel('Fraction','fontsize',16)
        title(['Evolution of WMtoC3: Session ' num2str(s) ''],'fontsize',16)
        hold off
    end

    % Label WEtoE figure
    if coll
        if guiFlag == 0
            figure(fig0+2)
            xlabel('Weight','fontsize',16)
            ylabel('Lap','fontsize',16)
            zlabel('Fraction','fontsize',16)
            title(['Evolution of WEtoE: Session ' num2str(s) ''],'fontsize',16)
            hold off
        elseif plotFlag == 4
            axes(handles.axes_main)
            xlabel('Weight','fontsize',16)
            ylabel('Lap','fontsize',16)
            zlabel('Fraction','fontsize',16)
            title(['Evolution of WEtoE: Session ' num2str(s) ''],'fontsize',16)
            hold off
        end
        
    end % if coll

    % Label WMtoC3 Lap 1 figure
    if lap1plot
        figure(fig0+3)
        xlabel('Weight','fontsize',16)
        ylabel('Angle (deg)','fontsize',16)
        zlabel('Fraction','fontsize',16)
        title(['Evolution of WMtoC3: Lap 1, Session ' num2str(s) ''],'fontsize',16)
    end

    % Reset for each session
    mechistmean = 0*mechistmean;
    ca3eSum = 0*ca3eSum;
    ca3iSum = 0*ca3iSum;

end %for sessions

cd ..
return

%%
function shist = ghist(dat,dim)

shist = zeros(dim,360);
rind = find(dat < -0.5);
rind(end+1) = rind(end)+length(dat(rind(end):end));
st = 2;

for j=1:360,

    fin = rind(j+1)-1;
    cind = dat(st:fin);
    shist(:,j) = 0;

    for k=1:length(cind)
        shist(cind(k),j) = shist(cind(k),j) + 1;
    end

    st = rind(j+1)+1;


end