%
% nethistFormRate(dname,sess,laps,Ne_M,Ne_C3,Pcells,wMax)
%
% Plots average spike rate for each cell in Pcells on the circular and
% linear tracks.
%
% Input: dname = name of directory created in hippomat2grid.m
%        sess = number of sessions
%        laps = number of laps
%        Ne_M = number of MEC cells
%        Ne_C3 = number of CA3 E cells
%        Pcells = place cells to be plotted (col vector); if Pcells is
%           empty, then it'll evaluate the place fields and return the place
%           cells and their field centers
%        wMax = max weight for WMtoC3
%        handles = handles from gui; empty matrix is not using the gui
%  
% Ouput: PlaceCells = indeces of place cells
%        centers = center of each place field

%%
function [PlaceCells,centers] = nethistFormRate(dname0,sess,laps,Ne_M,Ne_C3,Pcells,wMax,handles)

persistent dname

if ~isempty(dname0)
    dname = dname0;
end

cd(dname)

if ~isempty(handles)  %using gui
    guiFlag = 1;
else                  %not using gui
    guiFlag = 0;  
end

fig0 = 0;

if isempty(Pcells)
    action = 1;  %evaluate place fields
else
    action = 2;  %plot place fields
end

% Set up variables used to compute the spike rates
beta = 10e-3;  %kHz, should be the same as used in hippomat2grid.m
alpha = 100;  %ms, should be the same as used in hippomat2grid.m

dspkVec = 50*(14:-1:0)';  %ms
expVec = exp(-dspkVec/alpha);

% Initiate storage for rates
if action == 1
    Ncells = Ne_C3;
    cells = 1:Ne_C3;  %cells of interest
else
    Ncells = length(Pcells);
    cells = Pcells;  %cells of interest
end

rateT = zeros(Ncells,360);  %total rate for each session
rate = rateT;               %spike rate for each lap

for s = 1:sess

    for ell = 1:laps

    % Get CA3-E spike histogram
    str = [num2str(s) '-lap-' num2str(ell)];
    stre = ['ca3e-data-ses-' str];
    a = fopen(stre,'r');
    dat = fread(a,'int16');
    fclose(a);
    ca3ehist = ghist(dat,Ne_C3);  %Ne_C3 x 360
    
    % Compute the spike rate for each cell of interest during lap ell
    rate = 0*rate;  %spike rate for each lap
    for deg = 1:360
        if deg <= 14
            if ell == 1
                rate(:,deg) = beta*ca3ehist(cells,1:deg)*expVec(end-deg+1:end);
            else
                rate(:,deg) = beta*ca3ehist(cells,[(346+deg):360,1:deg])*expVec;
            end
        else
            rate(:,deg) = beta*ca3ehist(cells,(deg-14):deg)*expVec;
        end
    end  %for deg
       
    rateT = rateT + rate;
    
    end  %for laps

    rateAve = rateT/laps;  %Ncells x 360

    figs = (s-1)*(1+Ncells);  %number of figures used so far
    
    if guiFlag == 0
        % Plot firing fields on linear track
        figure(fig0+figs+1)
        imagesc(rateAve'*1e3)
        xlabel('CA3 E Cells','fontsize',16)
        ylabel('Angle (deg)','fontsize',16)
        title(['Average Spike Rate (Hz), Session ' num2str(s)''],'fontsize',16)
        colorbar
    end
    
    if action == 2  %plot spike rate for each cell in Pcells on circular track

        % Create matrices for storing rates as a function of position
        r = [31;35];
        thvec = (1:360)*pi/180;
        X = r*cos(thvec);
        Y = r*sin(thvec);

        % Without doing this, there's a hold in the plot for some reason
        X = [r X];
        Y = [zeros(2,1) Y];

        for k = 1:Ncells
            cell = Pcells(k);
            Rcell = rateAve(k,:);  %1 x 360

            C = zeros(2,360);
            angs = find(Rcell);
            for n = 1:length(angs)
                ang = angs(n);
                C(:,ang) = Rcell(ang);  %spike rate (Hz)
            end

            C = [C(:,end) C];

            figure(figs+1+k)
            pcolor(X,Y,C*1e3)
            axis equal tight
            axis off
            colorbar
            title(['Average Spike Rate (Hz), cell ' num2str(cell) ', Session ' num2str(s) ''],'fontsize',14)

        end  %for Ncells

    end %if plFlag
    
    % Reset variables
    rateT = 0*rateT; % (rate gets reset each lap)
    
end %for sess


%%
% Evaluate place fields
[places,flags] = findPlace(rateAve);

% Print results to screen
if action == 1  %(evaluating)
    
    Nfail0 = length(find(flags==0));
    Nfail1 = length(find(flags==-1));
    Nplace = length(find(flags==1));

    Pplace = Nplace/Ne_C3*100;
    Pfail0 = Nfail0/Ne_C3*100;
    Pfail1 = Nfail1/Ne_C3*100;

    if guiFlag == 0
        fprintf('Percentage of place cells: %3.2f%%, no firing fields: %3.2f%%, multiple firing fields: %3.2f%%\n', ...
            Pplace,Pfail0,Pfail1)
    else
        axes(handles.axes_top)
        bar(1:3,[Pplace,Pfail0,Pfail1])
        ylabel('Percentage of Cells','fontsize',16)
        title('Place Field Formation','fontsize',16)
        set(gca,'XTickLabel',{'Place Fields','No Fields','Multiple Fields'},'fontsize',16)
        cd ..
        return
    end
    
    PlaceCells = find(places);
    centers = places(PlaceCells);

else  %(plotting)

    for c = 1:Ncells
        cell = Pcells(c); 
        fprintf('cell: %i, place: %4.1f, flag: %i\n', cell,places(c),flags(c))
    end
    
end %if action
    
    
%%
% Show MEC input at the beginning and at the end
if action == 2

    mechistmean = zeros(Ne_M,360);
    
    for s = 1:sess
        
        % Load initial and final WMtoC3 (Ne_M x Ne_C3)
        W1 = load(['WMtoC3-init-ses-' num2str(s)]);
        W2 = load(['WMtoC3-fin-ses-' num2str(s)]);

        % Convert to matrix form (here, row = pre, col = post)
        WMtoC30 = spalloc(Ne_M,Ne_C3,ceil(.3*Ne_M*Ne_C3));
        WMtoC3F = WMtoC30;
        for k=1:size(W2,1)
            WMtoC30(W1(k,1),W1(k,2)) = W1(k,3);
            WMtoC3F(W2(k,1),W2(k,2)) = W2(k,3);
        end

        % collect MEC spike histograms
        mechistmean = 0*mechistmean;

        for ell = 1:laps
            st = ell*40+1;

            str = [num2str(s) '-lap-' num2str(ell)];
            strm = ['mec-data-ses-' str];
            a = fopen(strm,'r');
            dat = fread(a,'int16');
            fclose(a);
            mechist = ghist(dat,Ne_M);
            mechistmean = mechistmean + mechist;

        end %for laps

        mechistmean = mechistmean/laps;  % Ne_M x 360 (average number of spikes per degree for each cell)

        % Compute the average spike rate for each MEC cell
        MECrate = zeros(Ne_M,360);
        for deg = 1:360

            if deg <= 14
                if ell == 1
                    MECrate(:,deg) = beta*mechistmean(:,1:deg)*expVec(end-deg+1:end);
                else
                    MECrate(:,deg) = beta*mechistmean(:,[(346+deg):360,1:deg])*expVec;
                end
            else
                MECrate(:,deg) = beta*mechistmean(:,(deg-14):deg)*expVec;
            end

        end  %for deg
        
        % Plot MEC input and output
        % input = rate(MEC cell) * weight(MEC cell to CA3 cell)
    
        figs = s*(1+Ncells);  %number of figures used so far
        
        for c = 1:Ncells
            
            cell = Pcells(c);
            
            Wcell0 = WMtoC30(:,cell);  %initial weights from MEC cells to CA3E cell
            WcellF = WMtoC3F(:,cell);    %final weights from MEC cells to CA3E cell
            pres = find(Wcell0);       %presynaptic MEC cells to CA3E cell
    
            figure(figs+c)

            % Compute initial input (weighted rate sum for each degree)
            input0 = zeros(360,1);
            for deg = 1:360
                input0(deg) = sum(MECrate(pres,deg) .* Wcell0(pres), 1);
            end
                
            % Plot initial input
            subplot(1,4,1)
            imagesc(input0*1e3)
            colorbar
            title('Initial Input','fontsize',14)
            ylabel('Angle (deg)','fontsize',14)
    
            % Compute final input (weighted rate sum for each degree)
            inputF = 0*input0;
            for deg = 1:360
                inputF(deg) = sum(MECrate(pres,deg) .* WcellF(pres), 1);
            end
            
            % Plot final input
            subplot(1,4,2)
            imagesc(inputF*1e3)
            colorbar
            title('Final Input','fontsize',14)
            axis off
    
            % Plot CA3E rate for cell
            subplot(1,4,4)
            Rcell = rateAve(c,:);
            imagesc(Rcell'*1e3)
            colorbar
            title(['Average Rate (Hz), Cell ' num2str(cell)],'fontsize',14)
            axis([.5 1.5 1 360])
  
        end %for Ncells
       
    end  %for sess
    
end  %if ~isempty(Pcells)

cd ..

return


%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Find number of spikes for each degree for each cell
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

function shist = ghist(dat,dim)

shist = zeros(dim,360);
rind = find(dat < -0.5);
rind(end+1) = rind(end)+length(dat(rind(end):end));
st = 2;

for j=1:360,

    fin = rind(j+1)-1;
    cind = dat(st:fin);
    shist(:,j) = 0;

    for k=1:length(cind)
        shist(cind(k),j) = shist(cind(k),j) + 1;
    end

    st = rind(j+1)+1;


end


%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Evaluate place cells
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

function [place, flag]= findPlace(R)
%
% input: Rcell: average spike rate of the cell (Hz)  (Ne_C3 x 360)
% output: place: place field of the cell; 0 means it fails to meet the
% place field criteria
%         flag: 1: have a place cell
%               0: firing rate is too low
%               -1: multiple firing rates
%
%Criteria: place = location of max firing rate
%          max firing rate must be at least 3 Hz
%          outside of place +- 10 degrees, cannot have a rate greater than
%          1/3 its max rate
%

%R = Ncellsx360

[maxR,Mvec] = max(R,[],2);  %Mvec = degree of maximal spiking for each cell, 1 x Ncells
Ncells = size(R,1);

Rtog = [R(:,341:360) R R(:,1:20)];  %1 x 400

tol = 1e-5;

place = zeros(1,Ncells);
flag = place;
for cell = 1:Ncells
    if maxR(cell) >= 3e-3  %strong firing field (larger than 3 Hz)
 
        Rcell = R(cell,:);
        
        M = Mvec(cell);
        Mtog = M + 20;

        ind2 = find(Rtog(cell,Mtog+1:end)<tol,1)+M;
        ind1 = find(Rtog(cell,1:Mtog-1)<tol,1,'last')-20;
        
        if ind2 > 360
            ind3 = mod(ind2,360);
        elseif ind1<=0
            ind0 = 360+ind1;
        end

        rem = setdiff(1:360,ind1:ind2);
        if any(Rcell(rem) > maxR(cell)/3)  %multiple firing fields
            flag(cell) = -1;
        else  %single firing field larger than 3 Hz
%             place(cell) = M;
         
            if ind2>360
                degs = [ind1:360 1:ind3];
            elseif ind1 <= 0
                degs = [ind0:360 1:ind2];
            else
                degs = ind1:ind2;
            end
            
            Mass = sum(Rcell(degs));
            s = 0;
            
            for deg = degs
                s = s + deg*Rcell(deg);
            end

            com = s/Mass;
            place(cell) = com;
            
            flag(cell) = 1;
        end
    end
end



